/********************************************************************/
/* DRShowKeyRecMode:                                                */
/* Simple Xlib-based GUI app for showing                            */
/* the current or the default Keyboard-Recording mode.              */
/* When the modes change,                                           */
/* their displayed values are automatically updated.                */
/********************************************************************/

#define DR_CLIENT_ONLY
#include <X11/Xlib.h>
#include "DispRec/DispRecCtrlExtension.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define  COMPRESS

void usage()
{
	fprintf(stderr, "%s", 
		"DRGetKeyRecMode options:\n"
		"\t--get-default\n"
	);
}

void fatalerror(const char *msg)
{
	fprintf(stderr, "DRGetKeyRecMode: Fatal Error: %s\n", msg);

	usage();
	exit(1);
}

/* global state */
static Display* dpy = NULL;
static int DispRecCtrl_event_base, DispRecCtrl_error_base;
static enum XDispRecCtrl_KeyRecMode currentMode = XDispRecCtrl_KeyRecMode_Disabled;
static enum XDispRecCtrl_KeyRecMode defaultMode = XDispRecCtrl_KeyRecMode_Disabled;
static Window mainWindow = None;
static unsigned long foregroundPixel = 1;

/* GUI-functions */
static void Redraw()
{
	XGCValues gcAttr;
	GC gc;

	gcAttr.foreground = foregroundPixel;
	gc = XCreateGC( dpy, mainWindow, GCForeground, &gcAttr );

	XClearWindow( dpy, mainWindow );

	{
		char currentStateString[1024];
		sprintf( 
			currentStateString, 
			"current keyrec-mode = %s",
			XDispRecCtrl_KeyRecMode_To_String( currentMode )
		);

		XDrawString( 
			dpy, mainWindow, gc, 5, 20, 
			currentStateString, strlen(currentStateString) 
		);
	}

	{
		char defaultStateString[1024];
		sprintf( 
			defaultStateString, 
			"default keyrec-mode = %s",
			XDispRecCtrl_KeyRecMode_To_String( defaultMode )
		);

		XDrawString( 
			dpy, mainWindow, gc, 5, 40, 
			defaultStateString, strlen(defaultStateString) 
		);
	}

}

static void ProcessDispRecNotifyEvent( XDispRecCtrl_NotifyEvent *ev ) 
{
	switch ( ev->subtype ) {
		case DispRecCtrl_NotifyKeyRecModeChanged: {
			XDispRecCtrl_NotifyKeyRecModeChangedEvent* myEvent =
				(XDispRecCtrl_NotifyKeyRecModeChangedEvent*)ev;
			/* update the state */
			currentMode = myEvent->new_mode;
			/* draw new state */
			Redraw();

			/* debug output */
			#if 0			
			printf("currentChanged\n");
			fflush(stdout);
			#endif
		} break;
		case DispRecCtrl_NotifyDefaultKeyRecModeChanged: {
			XDispRecCtrl_NotifyDefaultKeyRecModeChangedEvent* myEvent =
				(XDispRecCtrl_NotifyDefaultKeyRecModeChangedEvent*)ev;
			/* update the state */
			defaultMode = myEvent->new_default_mode;
			/* draw new state */
			Redraw();

			/* debug output */
			#if 0			
			printf("defaultChanged\n");
			fflush(stdout);
			#endif
		} break;
	}
}

static void EventLoop()
{
	while(1) {
		XEvent event;
		XNextEvent( dpy, &event );
		switch( event.type ) {
			case Expose: {	
				Redraw();
			} break;
			default: {
				if ( event.type == DispRecCtrl_event_base + DispRecCtrl_Notify ) {
					ProcessDispRecNotifyEvent( 
						(XDispRecCtrl_NotifyEvent*) &event 
					);
				}
			} 
		}
	}
}

/* main */
int main(int argc, char* argv[])
{
	Bool haveDispRecCtrl=False;

	/* parse commandline */
	{
		int argIdx;
		for ( argIdx = 1; argIdx < argc; argIdx++ ) {
			const char* arg = argv[argIdx];
			{
				char* errorMsg = malloc( strlen(arg) + 100 ) ;
				sprintf( errorMsg, "illegal commandline arg %s", arg );
				fatalerror( errorMsg );
			}
		}
	}


	/* open display and check for DispRecCtrl Extension */
	dpy=XOpenDisplay(NULL);

	if ( dpy == NULL ) {
		fatalerror("Could not open the display.\n");
	}

	haveDispRecCtrl = 
		XDispRecCtrl_QueryExtension(
			dpy, 
			&DispRecCtrl_event_base, 
			&DispRecCtrl_error_base
		);

	if (!haveDispRecCtrl) {
		fatalerror("Display does not support the DispRecCtrl Extension\n");
	}

	/* compute foreground color pixel (white) */
	{
		XColor foreground;
		foreground.red = 0xffff;
		foreground.green = 0xffff;
		foreground.blue = 0xffff;
		foreground.flags = DoRed | DoGreen | DoBlue;
		XAllocColor( 
			dpy, 
			DefaultColormap( dpy, DefaultScreen(dpy) ), 
			&foreground 
		);
		foregroundPixel = foreground.pixel;
	}

	/* create the main window */
	mainWindow = 
		XCreateSimpleWindow(
			dpy,
			DefaultRootWindow(dpy),
			0, 0, /* width */ 300, /* height */ 50, 0, 0, 0
		);

	XMapWindow(dpy, mainWindow);
		
	/* select events */
	XSelectInput( dpy, mainWindow, ExposureMask );
	XDispRecCtrl_SelectInput( dpy, DispRecCtrl_NotifyMask );

	/* get initial KeyRec-modes */
	currentMode = XDispRecCtrl_GetKeyRecMode( dpy, 0 );
	defaultMode = XDispRecCtrl_GetKeyRecMode( dpy, XDispRecCtrl_KeyRecModeFlag_GetDefault );

	/* event-loop */
	EventLoop();

	/* finish */
	XCloseDisplay( dpy );
    return 0;
}

